==> Prerelease-5.16.tar.gz: rejui/src/JUI/Widgets/Window.cpp <==
#include "JUI/Widgets/Window.hpp"
#include <jlog/Logger.hpp>
#include <cstdlib>


namespace JUI {
    Window::Window() : Widget(), Clickable(), Hoverable(), RectBase(), Draggable(), Resizable(), Dockable() {
        Name("Window");
        this->Position({200, 200, 0, 0});
        this->Size({400, 200, 0, 0});
        min_size = {400, 200};
        this->BGColor(Style::Window::OutlineColor);
        this->BorderColor(Style::Window::OutlineColor);
        this->BorderWidth(Style::Window::OutlineWidth);
        this->BorderMode(BorderMode::Middle);

        // TODO: Move out of Event callback
        this->OnReleaseEvent += [&] (Vector2 dummy, MouseButton btn, bool dummy3) {
            if (dragging) {
                SetDrag(false);
            }
        };

        Viewport = new Rect(this);
        Viewport->Name("Viewport");
        Viewport->BGColor(Style::Window::ViewportBackgroundColor);
        Viewport->Size({0, -titlebar_height, 1, 1});
        Viewport->Position({0, titlebar_height, 0, 0});
        // TODO: Viewport->AnchorPoint({0.f, 0.f});
        Viewport->BorderColor({128, 128, 128, 255});
        Viewport->BorderWidth(0);
        //Viewport->Padding(1_px);

        // TODO: Viewport->SetBorderRadius(0);

        Topbar = new Rect(this);
        Topbar->Position({0_px, 0_px});
        Topbar->Size({100_percent, UDim(titlebar_height, 0)});
        //Topbar->BGColor({92,92,192, 255});
        Topbar->BGColor(Colors::Transparent);
        Topbar->BorderColor({128, 128, 128, 0});
        Topbar->BorderWidth(0);

        TitleLabel = new Text(Topbar);
        TitleLabel->Center();
        TitleLabel->SetContent(title);
        TitleLabel->SetTextSize(title_font_size);

        // TODO: Pull out this circle image generation code, make a Circle widget.

        // TODO: auto* list = new HorizontalListLayout(Topbar);
        // TODO: exit_btn

        auto* exb_tex = new Texture( Vector2i(titlebar_height, titlebar_height));
        auto* exb_rt = new RenderTarget(exb_tex);

        if (exb_rt->SetMSAAEnabled(JGL::SampleRate::X8)) {/* using msaa to make the circle nicer. */}
        std::array<Color4, 2> exb_circle_colors {
            Colors::White,//Color4(168, 28, 28, 255),
            Colors::White//Color4(212, 25, 25, 255)
        };

        std::array<Vector2, 2> exb_circle_positions {
                Vector2(exb_rt->GetDimensions()) / 2,
                Vector2(exb_rt->GetDimensions()) / 2
        };

        std::array<float, 2> exb_circle_radii {
                (float) (exb_rt->GetDimensions().x * 0.33),
                (float) (exb_rt->GetDimensions().x * 0.28)
        };

        J2D::Begin(exb_rt, true);
            J2D::BatchFillCircle(exb_circle_colors.data(), exb_circle_positions.data(), exb_circle_radii.data(), 24, 2);
        J2D::End();
        delete exb_rt;

        exit_btn = new ImageButton(Topbar);
        exit_btn->Content(exb_tex);
        //exit_btn->AnchorPoint({1.f, 0.f});
        //exit_btn->Size({30_px, 100_percent});
        exit_btn->Size({titlebar_height, titlebar_height, 0, 0});

        exit_btn->BGColor(Colors::Transparent);
        exit_btn->BGColors(Colors::Transparent, Colors::Transparent, Colors::Transparent, Colors::Transparent);
        exit_btn->BorderWidth(0.f);
        exit_btn->BaseImageColor(Colors::Reds::LightCoral);
        exit_btn->HoveredImageColor(Colors::Red);
        exit_btn->PressedImageColor(Colors::Reds::DarkRed);

        exit_btn->OnReleaseEvent += [&] (auto... _) {
            this->Visible(false);
        };
    }

    Window::Window(Widget* parent) : Window() {
        this->Parent(parent);
    }

    Vector2 Window::MinSize() const { return min_size; }

    Vector2 Window::MaxSize() const { return max_size; }

    Vector2 Window::CurrentSize() const { return this->GetAbsoluteSize(); }

    std::string Window::Title() const { return title; }

    bool Window::IsDraggable() const { return draggable; }

    bool Window::IsDockable() const { return dockable; }

    bool Window::IsResizable() const { return resizable; }

    Text *Window::TitleInstance() { return TitleLabel; }

    Rect *Window::TopbarInstance() { return Topbar; }

    Rect *Window::ViewportInstance() { return Viewport; }

    void Window::SetDrag(bool d) {
        Draggable::SetDrag(d);
        this->initial_drag_offset = this->last_known_mouse_pos - this->GetAbsolutePosition();
    }

    void Window::SetTitleFont(const Font& f) {
        TitleLabel->SetFont(f);
        //exit_btn->SetFont(f);
    }

    void Window::OnClick(const Vector2& mouse_pos, const MouseButton& btn)
    {
        Clickable::OnClick(mouse_pos, btn);
        clicked = true;

        if (draggable && btn == MouseButton::Left && TitleInstance()->IsMouseInside())
            this->SetDrag(true);

        if (resizable && btn == MouseButton::Right && TitleInstance()->IsMouseInside()) {
            this->StartResizing(last_known_mouse_pos);
            this->SetResize(true);
            size_when_restart_began = Size();
        }
    }

    void Window::OnRelease(const J3ML::LinearAlgebra::Vector2 &mouse_pos, const JUI::MouseButton &btn,
                           bool still_hovering) {
        Clickable::OnRelease(mouse_pos, btn, still_hovering);
        clicked = false;
        if (dragging)
            this->SetDrag(false);

        if (resizing)
            this->SetResize(false);
    }


    void Window::DragTo(const Vector2& pos)
    {
        this->Position(UDim2{(int) pos.x, (int) pos.y, 0, 0});
    }

    void Window::Update(float delta) {
        if (dragging) {
            // Move the window with the mouse, accounting for initial offset of window-mouse when we first clicked.
            Vector2 goal = last_known_mouse_pos - initial_drag_offset;
            DragTo(goal);
        }

        if (resizing) {
            Vector2 mpos = this->last_known_mouse_pos;

            Vector2 resize_amt = mpos - initial_resize_offset;

            Size(size_when_restart_began + UDim2(resize_amt.x, resize_amt.y, 0, 0));

            Vector2 abs_size = this->GetAbsoluteSize();

            // Clamp X-axis.
            if (abs_size.x < min_size.x)
            {
                float needed_amt_to_resize = min_size.x - abs_size.x;
                Size(Size() + UDim2(needed_amt_to_resize, 0, 0, 0));
            }

            // Clamp Y-axis.
            if (abs_size.y < min_size.y)
            {
                float needed_amt_to_resize = min_size.y - abs_size.y;
                Size(Size() + UDim2(0, needed_amt_to_resize, 0, 0));
            }
        }

        Widget::Update(delta);
        hovered = IsMouseInside();

        Hoverable::Update(last_known_mouse_pos, delta);

        // TODO: This is duplicated between here and Window.cpp
        // Will be abstracted away into Clickable shortly.
        if (IsHovered() && mb_state && !prev_mb_state)
        {
            OnClick(last_known_mouse_pos, mbtn);
        }

        if (IsClicked() && !mb_state)
        {
            OnRelease(last_known_mouse_pos, mbtn, IsHovered());
        }

        prev_mb_state = mb_state;

    }
    void Window::Draw() {
        if (!visible) // Duped from Rect.cpp::Draw...
            return;
        RectBase::Draw(this->GetAbsolutePosition(), this->GetAbsoluteSize());
        Widget::Draw();
    }

    void Window::SetTitle(const std::string &title) {
        TitleInstance()->SetContent(title);
    }

    void Window::MinSize(const Vector2 &constraint) {
        min_size = constraint;
    }

    void Window::MaxSize(const Vector2 &constraint) {
        max_size = constraint;
    }

    void Window::SetDraggable(bool value) {
        draggable = value;
    }

    void Window::SetDockable(bool value) {
        dockable = value;
    }

    void Window::SetResizable(bool value) {
        resizable = value;
    }

    ImageButton* Window::ExitButtonInstance() {
        return exit_btn;
    }

    void Window::LayoutControlsLeft() {
        exit_btn->AnchorPoint({0.f, 0.f});
        exit_btn->Position({0_px, 0_px});
    }

    void Window::LayoutControlsRight() {
        exit_btn->AnchorPoint({1.f, 0.f});
        exit_btn->Position({100_percent, 0_px});
    }

    Vector2 Window::AbsoluteViewportPosition() const {
        return Viewport->GetAbsolutePosition();
    }

    Vector2 Window::AbsoluteViewportSize() const {
        return Viewport->GetAbsoluteSize();
    }

    AABB2D Window::AbsoluteViewportBounds() const {
        return {AbsoluteViewportPosition(), AbsoluteViewportSize()};
    }

    void Window::UpdateInternalWidgetsTitlebarHeight() {
        Viewport->Size({0, -titlebar_height, 1, 1});
        Viewport->Position({0, titlebar_height, 0, 0});

        Topbar->Size({100_percent, UDim(titlebar_height, 0)});
        exit_btn->Size({titlebar_height, titlebar_height, 0, 0});
    }

    void Window::TitlebarHeight(int height) {
        titlebar_height = height;
        UpdateInternalWidgetsTitlebarHeight();
    }

    int Window::TitlebarHeight() const {
        return titlebar_height;
    }

    void Window::CornerRounding(float radius) {
        RectBase::CornerRounding(radius);
    }

    void Window::SetOpen(bool value) {
        open = value;
        Visible(open);
    }

    void Window::Toggle() {
        SetOpen(!IsOpen());
    }

    void Window::Close() { SetOpen(false); }

    void Window::Open() { SetOpen(true);  }

    bool Window::IsOpen() const { return open;}
}

==> Prerelease-6.1.tar.gz: rejui/src/JUI/Widgets/Window.cpp <==
#include "JUI/Widgets/Window.hpp"
#include <jlog/Logger.hpp>
#include <cstdlib>


namespace JUI {
    Window::Window() : Widget(), Clickable(), Hoverable(), RectBase(), Draggable(), Resizable(), Dockable() {
        Name("Window");
        this->Position({200, 200, 0, 0});
        this->Size({400, 200, 0, 0});
        min_size = {400, 200};
        this->BGColor(Style::Window::OutlineColor);
        this->BorderColor(Style::Window::OutlineColor);
        this->BorderWidth(Style::Window::OutlineWidth);
        this->BorderMode(BorderMode::Middle);

        // TODO: Move out of Event callback
        this->OnReleaseEvent += [&] (Vector2 dummy, MouseButton btn, bool dummy3) {
            if (dragging) {
                SetDrag(false);
            }
        };

        Viewport = new Rect(this);
        Viewport->Name("Viewport");
        Viewport->BGColor(Style::Window::ViewportBackgroundColor);
        Viewport->Size({0, -titlebar_height, 1, 1});
        Viewport->Position({0, titlebar_height, 0, 0});
        // TODO: Viewport->AnchorPoint({0.f, 0.f});
        Viewport->BorderColor({128, 128, 128, 255});
        Viewport->BorderWidth(0);
        //Viewport->Padding(1_px);

        // TODO: Viewport->SetBorderRadius(0);

        Topbar = new Rect(this);
        Topbar->Position({0_px, 0_px});
        Topbar->Size({100_percent, UDim(titlebar_height, 0)});
        //Topbar->BGColor({92,92,192, 255});
        Topbar->BGColor(Colors::Transparent);
        Topbar->BorderColor({128, 128, 128, 0});
        Topbar->BorderWidth(0);

        TitleLabel = new Text(Topbar);
        TitleLabel->Center();
        TitleLabel->Content(title);
        TitleLabel->TextSize(title_font_size);

        // TODO: Pull out this circle image generation code, make a Circle widget.

        // TODO: auto* list = new HorizontalListLayout(Topbar);
        // TODO: exit_btn

        auto* exb_tex = new Texture( Vector2i(titlebar_height, titlebar_height));
        auto* exb_rt = new RenderTarget(exb_tex);

        if (exb_rt->SetMSAAEnabled(JGL::SampleRate::X8)) {/* using msaa to make the circle nicer. */}
        std::array<Color4, 2> exb_circle_colors {
            Colors::White,//Color4(168, 28, 28, 255),
            Colors::White//Color4(212, 25, 25, 255)
        };

        std::array<Vector2, 2> exb_circle_positions {
                Vector2(exb_rt->GetDimensions()) / 2,
                Vector2(exb_rt->GetDimensions()) / 2
        };

        std::array<float, 2> exb_circle_radii {
                (float) (exb_rt->GetDimensions().x * 0.33),
                (float) (exb_rt->GetDimensions().x * 0.28)
        };

        J2D::Begin(exb_rt, true);
            J2D::BatchFillCircle(exb_circle_colors.data(), exb_circle_positions.data(), exb_circle_radii.data(), 24, 2);
        J2D::End();
        delete exb_rt;

        exit_btn = new ImageButton(Topbar);
        exit_btn->Content(exb_tex);
        //exit_btn->AnchorPoint({1.f, 0.f});
        //exit_btn->Size({30_px, 100_percent});
        exit_btn->Size({titlebar_height, titlebar_height, 0, 0});

        exit_btn->BGColor(Colors::Transparent);
        exit_btn->BGColors(Colors::Transparent, Colors::Transparent, Colors::Transparent, Colors::Transparent);
        exit_btn->BorderWidth(0.f);
        exit_btn->BaseImageColor(Colors::Reds::LightCoral);
        exit_btn->HoveredImageColor(Colors::Red);
        exit_btn->PressedImageColor(Colors::Reds::DarkRed);

        exit_btn->OnReleaseEvent += [&] (...) {
            this->Visible(false);
        };
    }

    Window::Window(Widget* parent) : Window() {
        this->Parent(parent);
    }

    Vector2 Window::MinSize() const { return min_size; }

    Vector2 Window::MaxSize() const { return max_size; }

    Vector2 Window::CurrentSize() const { return this->GetAbsoluteSize(); }

    std::string Window::Title() const { return title; }

    bool Window::IsDraggable() const { return draggable; }

    bool Window::IsDockable() const { return dockable; }

    bool Window::IsResizable() const { return resizable; }

    Text *Window::TitleInstance() { return TitleLabel; }

    Rect *Window::TopbarInstance() { return Topbar; }

    Rect *Window::ViewportInstance() { return Viewport; }

    Rect * Window::Content() { return ViewportInstance(); }

    void Window::SetDrag(bool d) {
        Draggable::SetDrag(d);
        this->initial_drag_offset = this->last_known_mouse_pos - this->GetAbsolutePosition();
    }

    void Window::SetTitleFont(const Font& f) {
        TitleLabel->Font(f);
        //exit_btn->SetFont(f);
    }

    void Window::OnClick(const Vector2& mouse_pos, const MouseButton& btn)
    {
        Clickable::OnClick(mouse_pos, btn);
        clicked = true;

        if (draggable && btn == MouseButton::Left && TitleInstance()->IsMouseInside())
            this->SetDrag(true);

        if (resizable && btn == MouseButton::Right && TitleInstance()->IsMouseInside()) {
            this->StartResizing(last_known_mouse_pos);
            this->SetResize(true);
            size_when_restart_began = Size();
        }
    }

    void Window::OnRelease(const J3ML::LinearAlgebra::Vector2 &mouse_pos, const JUI::MouseButton &btn,
                           bool still_hovering) {

        // TODO: This ONLY invokes if the mouse is still indeed **hovering**
        // Some controls would like to listen in cases where it isn't.

        Clickable::OnRelease(mouse_pos, btn, still_hovering);
        clicked = false;


        if (dragging)
            this->SetDrag(false);



        if (resizing) {
            this->SetResize(false);
        }
    }


    void Window::DragTo(const Vector2& pos)
    {
        this->Position(UDim2{(int) pos.x, (int) pos.y, 0, 0});
    }

    void Window::Update(float delta) {
        if (dragging) {
            // Move the window with the mouse, accounting for initial offset of window-mouse when we first clicked.
            Vector2 goal = last_known_mouse_pos - initial_drag_offset;
            DragTo(goal);
        }

        if (resizing) {
            Vector2 mpos = this->last_known_mouse_pos;

            Vector2 resize_amt = mpos - initial_resize_offset;

            Size(size_when_restart_began + UDim2(resize_amt.x, resize_amt.y, 0, 0));

            Vector2 abs_size = this->GetAbsoluteSize();

            // Clamp X-axis.
            if (abs_size.x < min_size.x)
            {
                float needed_amt_to_resize = min_size.x - abs_size.x;
                Size(Size() + UDim2(needed_amt_to_resize, 0, 0, 0));
            }

            // Clamp Y-axis.
            if (abs_size.y < min_size.y)
            {
                float needed_amt_to_resize = min_size.y - abs_size.y;
                Size(Size() + UDim2(0, needed_amt_to_resize, 0, 0));
            }
        }

        Widget::Update(delta);
        hovered = IsMouseInside();

        Hoverable::Update(last_known_mouse_pos, delta);

        // TODO: This is duplicated between here and Window.cpp
        // Will be abstracted away into Clickable shortly.
        if (IsHovered() && mb_state && !prev_mb_state)
        {
            OnClick(last_known_mouse_pos, mbtn);
        }


        if (prev_mb_state && !mb_state)
        {
            OnRelease(last_known_mouse_pos, mbtn, IsHovered());
        }

        prev_mb_state = mb_state;

    }
    void Window::Draw() {
        if (!visible) // Duped from Rect.cpp::Draw...
            return;
        RectBase::Draw(this->GetAbsolutePosition(), this->GetAbsoluteSize());
        Widget::Draw();

        //J2D::DrawString(Colors::Black, std::format("hover: {}", hovered), this->GetAbsolutePosition().x, this->GetAbsolutePosition().y, 1.f, 10);
        //J2D::DrawString(Colors::Black, std::format("mbutton: {}", mb_state), this->GetAbsolutePosition().x, this->GetAbsolutePosition().y+10, 1.f, 10);
    }

    void Window::Title(const std::string &title) {
        TitleInstance()->Content(title);
    }

    void Window::MinSize(const Vector2 &constraint) {
        min_size = constraint;
    }

    void Window::MaxSize(const Vector2 &constraint) {
        max_size = constraint;
    }

    void Window::SetDraggable(bool value) {
        draggable = value;
    }

    void Window::SetDockable(bool value) {
        dockable = value;
    }

    void Window::SetResizable(bool value) {
        resizable = value;
    }

    ImageButton* Window::ExitButtonInstance() {
        return exit_btn;
    }

    void Window::LayoutControlsLeft() {
        exit_btn->AnchorPoint({0.f, 0.f});
        exit_btn->Position({0_px, 0_px});
    }

    void Window::LayoutControlsRight() {
        exit_btn->AnchorPoint({1.f, 0.f});
        exit_btn->Position({100_percent, 0_px});
    }

    Vector2 Window::AbsoluteViewportPosition() const {
        return Viewport->GetAbsolutePosition();
    }

    Vector2 Window::AbsoluteViewportSize() const {
        return Viewport->GetAbsoluteSize();
    }

    AABB2D Window::AbsoluteViewportBounds() const {
        return {AbsoluteViewportPosition(), AbsoluteViewportSize()};
    }

    void Window::UpdateInternalWidgetsTitlebarHeight() {
        Viewport->Size({0, -titlebar_height, 1, 1});
        Viewport->Position({0, titlebar_height, 0, 0});

        Topbar->Size({100_percent, UDim(titlebar_height, 0)});
        exit_btn->Size({titlebar_height, titlebar_height, 0, 0});
    }

    void Window::TitlebarHeight(int height) {
        titlebar_height = height;
        UpdateInternalWidgetsTitlebarHeight();
    }

    int Window::TitlebarHeight() const {
        return titlebar_height;
    }

    void Window::CornerRounding(float radius) {
        RectBase::CornerRounding(radius);
    }

    void Window::SetOpen(bool value) {
        open = value;
        Visible(open);
    }

    void Window::Toggle() {
        SetOpen(!IsOpen());
    }

    void Window::Close() { SetOpen(false); }

    void Window::Open() { SetOpen(true);  }

    bool Window::ObserveMouseMovement(const Vector2 &latest_known_pos) {
        /// 1. If dragging or resizing, or check if any children are observing.

        if (!this->visible && !this->focused)
            return false;

        if (Widget::ObserveMouseMovement(latest_known_pos))
            return true;

        if (dragging || resizing)
            return true;




        return false;
    }

    bool Window::IsOpen() const { return open;}
}
